;
;                    Inertia Realtime 3D Rendering Engine
;    Copyright (c) 1996, Alex Chalfin, Jeroen Bouwens. All Rights Reserved.
;          DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED
;

;
; CULL.ASM
; code: Alex Chalfin
;

.486p                           ; 486 protected mode
.Model Flat,Pascal              ; with flat memory model
                                ;
                                ;
DATA SEGMENT PARA PUBLIC USE32  ;
                                ;
include strucdef.asm            ;  Inertia structure references
                                ;
Public CullPolys                ;
                                ;
_tmp_flt      dd ?              ;
                                ;
ENDS                            ;
                                ;
;================================================================
; Code Segment
;================================================================
                                ;
CODE SEGMENT PARA PUBLIC USE32  ;
                                ;
;===========================================================================
; Macros
;===========================================================================
                                ;
                                ;
;===========================================================================
; CullPolys
;===========================================================================
CullPolys Proc Uses edi esi ebx,Object : DWord, CullVector : DWord
                                ;
  Mov  edi,DWord Ptr [Object]   ;
  Mov  ecx,[edi].NumPoly        ;  Grab the number of polygons
                                ;
  Cmp  ecx,0                    ;  Check to see if there are any polygons
  Jle @@ExitNoCullPolygons      ;  to Cull
                                ;
  Mov  esi,[edi].Polygons       ;  Polyon pointer
  Mov  edi,DWord Ptr [CullVector];
                                ;
  fld  dword ptr [edi].X3D      ; culling vector components
  fld  dword ptr [edi].Y3D      ;
  fld  dword ptr [edi].Z3D      ; Store on the FPU stack for entire
                                ; culling duration
 @@CullLooper:                  ;
                                ;
  Mov   ebx,[esi].PolyAPtr      ; grab the pointer to the first vertex
                                ; in the polygon
                                ;
  fld   st(2)                   ; CullVector.x
  fsub  dword ptr [ebx].LocX    ; P^.A^.x
  fmul  dword ptr [esi].NormalX ; (CullVector.x - P^.A^.x) * P^.N.x
                                ;
  fld   st(2)                   ;
  fsub  dword ptr [ebx].LocY    ;
  fmul  dword ptr [esi].NormalY ; (CullVector.y - P^.A^.y) * P^.N.y
                                ;
  fld   st(2)                   ;
  fsub  dword ptr [ebx].LocZ    ;
  fmul  dword ptr [esi].NormalZ ; (CullVector.z - P^.A^.z) * P^.N.z
                                ;
  Xor    edx,edx                ; clear edx for later use
                                ;
  faddp st(1),st(0)             ;
  faddp st(1),st(0)             ;
                                ; st(0) now contains the final dot-product
                                ;
  fldz                          ;
  fcompp                        ; Cmp st(0),st(1) and pop both
  fstsw  ax                     ; store the status word (lame way to compare)
  sahf                          ; store FPU flags into GPR flags
                                ;
  Ja  @@PolygonNotVisible       ; we must use Ja or Jb because the
                                ; sign flag does not get set or something
  Inc  edx                      ; set the polygon to visible
                                ;
  Mov  edi,[esi].PolyBPtr       ; Get Vertex B ptr
  Mov  eax,[esi].PolyCPtr       ; Get Vertex C ptr
                                ;
  Mov  [ebx].VVisible,edx       ; Set Vertex A flag
  Mov  [edi].VVisible,edx       ; Set Vertex B flag
  Mov  [eax].VVisible,edx       ; Set Vertex C flag
                                ;
 @@PolygonNotVisible:           ;
                                ;
  Mov  [esi].PVisible,edx       ; Store the polygon visible flag
                                ;
  Add  esi,POLYGON_SIZE         ;
  Dec  ecx                      ;
  Jnz @@CullLooper              ;
                                ;
  fstp dword ptr [_tmp_flt]     ; clear the FPU
  fstp dword ptr [_tmp_flt]     ;
  fstp dword ptr [_tmp_flt]     ;
                                ;
 @@ExitNoCullPolygons:          ;
  Ret                           ;
CullPolys EndP                  ;


ENDS
END

Comment 
============================================================================
  Pascal Versions of each of the functions
============================================================================

Procedure CullPolys(Var Obj : VectorObject; Var CullVector : Vertex3d);

Var
  i : Longint;
  P : pTriangle;
  cvx : Single;

Begin
  For i := 0 to Obj.NumPoly - 1 do
    Begin
      P := @Obj.Polygons^[i];

      cvx :=  (CullVector.x - P^.A^.Loc.x) * P^.N.x +
              (CullVector.y - P^.A^.Loc.y) * P^.N.y +
              (CullVector.z - P^.A^.Loc.z) * P^.N.z;

      if (cvx >= 0.0)
        Then Begin
          P^.Visible    := 1;
          P^.A^.Visible := 1;
          P^.B^.Visible := 1;
          P^.C^.Visible := 1;
        End
        Else Begin
          P^.Visible := 0;
        End;
    End;
End;



