(*

                     Inertia Realtime 3D Rendering Engine
     Copyright (c) 1996, Alex Chalfin, Jeroen Bouwens. All Rights Reserved.
           DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED

*)

Unit Types;
{$I DEFINE.INC}

Interface

Type
  Real = Single;

Const
{$IFDEF SHAREWARE}
  MAXPOLYGONS = 500;
{$ENDIF}

  MAXLIGHTS = 8;      { up to 8 light sources                     }
  XLOC = 9;           { Index in matrix for object X location     }
  YLOC = 10;          { Index in matrix for object Y location     }
  ZLOC = 11;          { Index in matrix for object Z location     }
  PERSPECTIVE = 512;  { Perspective distance value for projection }
  FIXONE = 65536;     { Use 16.16 fixed point                     }
  MAXTEXTURES = 12;   { Maximum number of textures available at one time }
  POLYLISTSIZE = 2048;
  MAXPAGES = 8;       { maximum of 8 video pages }
  CORRECTRES = 128;

  I_OK         = $00;
  I_MEMERR     = $01;
  I_FILEERR    = $02;
  I_FORMATERR  = $03;

  AMBIENT      = $00;
  LAMBERT      = $01;
  GOURAUD      = $02;
  PHONG        = $03;
  UNSHADED     = $04;

  SMOOTH       = $00;
  TEXTURE      = $10;
  REFLECTION   = $20;
  PERSPTEXTURE = $30;

  TRANSPARENT  = $80;

  SHADE_MASK    = $0f;
  SURFACE_MASK  = $70;

  CLIP_Z         = $01;
  CLIP_LEFT      = $02;
  CLIP_RIGHT     = $04;
  CLIP_TOP       = $08;
  CLIP_BOTTOM    = $10;
  CLIP_GOURAUD   = $20;
  CLIP_TEXTURE   = $40;
  CLIP_ENV       = $80;

  CLIP_SHADEMASK = (CLIP_GOURAUD or CLIP_TEXTURE or CLIP_ENV);

Type
  RGBType = Record
    r, g, b : Byte;
  End;
  Palette = Array[0..255] of RGBType;

  Matrix4x3 = Array[0..11] of Single;

  tMap = Array[0..65535] of Byte;
  pMap = ^tMap;

  pTextureRec = ^TextureRec;
  TextureRec = Record
    Texture  : Pointer;
    Pal      : Palette;
  End;

  pVertex3d = ^Vertex3d;
  Vertex3d = Record       { 12 }
    x, y, z : Single;
  End;

  fVertex2d = Record        { 8 }
    x, y : Single;
  End;

  Vertex2d = Record       { 8 }
    x, y : Longint;
  End;

  pVertex = ^Vertex;
  Vertex = Record
    Loc      : Vertex3d;    { 12 }
    Cam      : Vertex3d;    { 12 }
    Scr      : fVertex2d;   { 8  }
    Env      : fVertex2d;   { 8  }
    N        : Vertex3d;    { 12 }
    OneOverZ : Single;      { 4  }
    Shade    : Single;      { 4  }
    Visible  : Longint;     { 4  }
  End;                      { 64 bytes per vertex }

  pTriangle = ^Triangle;
  Triangle = Record
    A, B, C :  pVertex;     { 12 }
    N        : Vertex3d;    { 12 }
    UV1,                    { 8  }
    UV2,                    { 8  }
    UV3      : fVertex2d;   { 8  }
    Texture  : Pointer;     { 4  }
    IBuffer  : Pointer;     { 4  }
    Shade    : Single;      { 4  }
    Visible  : Longint;     { 4  }
    Color    : Longint;     { 4  }
  End;                      { 68 bytes }

  tVertexList = Array[0..10000] of Vertex;
  pVertexList = ^tVertexList;
  tTriangleList = Array[0..10000] of Triangle;
  pTriangleList = ^tTriangleList;

  pSortRec = ^tSortRec;
  tSortRec = Record
    ZMid         : Longint;
    ShadeSurface : Longint;
    Next         : pSortRec;
    Poly         : pTriangle;
  End;

  tSortList = Array[0..40000] of tSortRec;
  pSortList = ^tSortList;

  tRenderList = Record
    Allocated : Longint;
    SortList  : pSortList;
    Count     : Longint;
    MSBList   : Array[0..255] of pSortRec;
    LSBList   : Array[0..255] of pSortRec;
  End;


  tLightList = Record
    NumLights : Longint;
    TempLight : Array[0..MAXLIGHTS-1] of Vertex3d;
    Light     : Array[0..MAXLIGHTS-1] of Vertex3d;
  End;

  VectorObject = Record
    NumVert           : Longint;            { 0 }
    NumPoly           : Longint;            { 2 }
    Vertexs           : pVertexList;        { 4 }
    Polygons          : pTriangleList;      { 4 }

    RenderMask        : Longint;
    Radius            : Single;
    CorrectCount      : Longint;
    ClipMask          : Longint;
    Cull              : Longint;

    Orientation       : Matrix4x3;
  End;

  ViewObject = Record
    CenterX : Single;
    CenterY : Single;

    WinMinX : Longint;
    WinMinY : Longint;
    WinMaxX : Longint;
    WinMaxY : Longint;

    InvCamZFac : Single;
    InvCamYFac : Single;
    CamZFac : Single;
    CamYFac : Single;

    FocalLength : Single;
    FieldOfView : Single;

    FustrumMinZ : Single;
    FustrumMaxZ : Single;

    Camera  : Matrix4x3;
  End;


  TFileBuffer = Array[0..1023] of Byte;
  PFileBuffer = ^TFileBuffer;

  ShaderFunction  = Procedure (Var View : ViewObject; Var Obj : VectorObject; M : Matrix4x3);
  PolygonFunction = Procedure (Poly : pTriangle; VPage : Pointer; X, Y : Longint);

Var
  Shaders : Array[0..255] of ShaderFunction;
  TriFillers : Array[0..255] of PolygonFunction;
  AspectRatio : Single;
  Lights : tLightList;

  GlobalPalette : Palette;
  IntensityTable : pMap;
  TransparencyTable : pMap;

  RenderList : tRenderList;

  TextureList : Array[0..MAXTEXTURES-1] of TextureRec;


Implementation


End.
